@@ -2,6 +2,7 @@
 Optimize
 ========
 
+Multistart optimization with support for various optimizers.
 """
 
 from .options import OptimizeOptions
@@ -10,6 +11,7 @@ from .optimize import (
 from .optimizer import (
     Optimizer,
     ScipyOptimizer,
+    IpoptOptimizer,
     DlibOptimizer,
     PyswarmOptimizer)
 from .result import OptimizerResult
