@@ -5,9 +5,13 @@ This is for testing the petab import.
 import os
 import unittest
 import numpy as np
+import pytest
 
+import amici
 import petab
 import pypesto
+import pypesto.optimize
+import pypesto.petab
 from test.petab_util import folder_base
 
 
@@ -30,7 +34,7 @@ class PetabImportTest(unittest.TestCase):
 
     def test_1_compile(self):
         for petab_problem in self.petab_problems:
-            importer = pypesto.PetabImporter(petab_problem)
+            importer = pypesto.petab.PetabImporter(petab_problem)
             self.petab_importers.append(importer)
 
             # check model
@@ -61,15 +65,57 @@ class PetabImportTest(unittest.TestCase):
         for obj_edatas, importer in \
                 zip(self.obj_edatas, self.petab_importers):
             obj = obj_edatas[0]
-            optimizer = pypesto.ScipyOptimizer(options={'maxiter': 10})
+            optimizer = pypesto.optimize.ScipyOptimizer(
+                options={'maxiter': 10})
             problem = importer.create_problem(obj)
-            result = pypesto.minimize(
+            result = pypesto.optimize.minimize(
                 problem=problem, optimizer=optimizer, n_starts=2)
 
             self.assertTrue(np.isfinite(
                 result.optimize_result.get_for_key('fval')[0]))
 
 
+def test_max_sensi_order():
+    """Test that the AMICI objective created via PEtab exposes derivatives
+    correctly."""
+    model_name = "Boehm_JProteomeRes2014"
+    problem = pypesto.petab.PetabImporter.from_yaml(
+        os.path.join(folder_base, model_name, model_name + '.yaml'))
+
+    # define test parameter
+    par = problem.petab_problem.x_nominal_scaled
+    npar = len(par)
+
+    # auto-computed max_sensi_order and fim_for_hess
+    objective = problem.create_objective()
+    hess = objective(par, sensi_orders=(2,))
+    assert hess.shape == (npar, npar)
+    assert (hess != 0).any()
+    objective.amici_solver.setSensitivityMethod(
+        amici.SensitivityMethod_adjoint)
+    with pytest.raises(ValueError):
+        objective(par, sensi_orders=(2,))
+    objective.amici_solver.setSensitivityMethod(
+        amici.SensitivityMethod_forward)
+
+    # fix max_sensi_order to 1
+    objective = problem.create_objective(max_sensi_order=1)
+    objective(par, sensi_orders=(1,))
+    with pytest.raises(ValueError):
+        objective(par, sensi_orders=(2,))
+
+    # do not use FIM
+    objective = problem.create_objective(fim_for_hess=False)
+    with pytest.raises(ValueError):
+        objective(par, sensi_orders=(2,))
+
+    # only allow computing function values
+    objective = problem.create_objective(max_sensi_order=0)
+    objective(par)
+    with pytest.raises(ValueError):
+        objective(par, sensi_orders=(1,))
+
+
 if __name__ == '__main__':
     suite = unittest.TestSuite()
     suite.addTest(PetabImportTest())
